Package: rbiodatacr
Type: Package
Title: R Client for the BIODATACR Biodiversity Data Platform of Costa
        Rica
Version: 0.1.1
Authors@R: person(
    "Manuel", "Spinola",
    email = "mspinola10@gmail.com",
    role  = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-7839-1908"))
Description: Provides functions to query occurrence records, species
    information, and datasets from BIODATACR
    <https://biodiversidad.go.cr>, the national biodiversity information
    platform of Costa Rica managed by the Technical Office of CONAGEBIO, Costa Rica.
    Built on the Atlas of Living Australia (ALA) API infrastructure.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/ManuelSpinola/rbiodatacr,
        https://manuelspinola.github.io/rbiodatacr/
BugReports: https://github.com/ManuelSpinola/rbiodatacr/issues
Suggests: CoordinateCleaner, ggplot2, knitr, rmarkdown, sf, testthat
        (>= 3.0.0)
Config/testthat/edition: 3
Imports: cli, dplyr, httr, jsonlite, purrr
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
LazyData: true
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-05 18:45:29 UTC; manuel_nuevo
Author: Manuel Spinola [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7839-1908>)
Maintainer: Manuel Spinola <mspinola10@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-05 22:30:16 UTC
Built: R 4.5.2; ; 2026-05-06 02:04:44 UTC; unix

Package: rbiodatacr
Type: Package
Title: R Client for the BIODATACR Biodiversity Data Platform of Costa
        Rica
Version: 0.1.1
Authors@R: person(
    "Manuel", "Spinola",
    email = "mspinola10@gmail.com",
    role  = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-7839-1908"))
Description: Provides functions to query occurrence records, species
    information, and datasets from BIODATACR
    <https://biodiversidad.go.cr>, the national biodiversity information
    platform of Costa Rica managed by the Technical Office of CONAGEBIO, Costa Rica.
    Built on the Atlas of Living Australia (ALA) API infrastructure.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/ManuelSpinola/rbiodatacr,
        https://manuelspinola.github.io/rbiodatacr/
BugReports: https://github.com/ManuelSpinola/rbiodatacr/issues
Suggests: CoordinateCleaner, ggplot2, knitr, rmarkdown, sf, testthat
        (>= 3.0.0)
Config/testthat/edition: 3
Imports: cli, dplyr, httr, jsonlite, purrr
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
LazyData: true
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-05 18:45:29 UTC; manuel_nuevo
Author: Manuel Spinola [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7839-1908>)
Maintainer: Manuel Spinola <mspinola10@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-05 22:30:16 UTC
Built: R 4.6.0; ; 2026-05-06 02:22:49 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bdcr_count)
export(bdcr_count_batch)
export(bdcr_occurrences)
export(bdcr_occurrences_batch)
export(bdcr_quality_check)
export(bdcr_species_search)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# rbiodatacr

# rbiodatacr <a href="https://manuelspinola.github.io/rbiodatacr/"><img src="man/figures/logo.png" align="right" height="139" alt="rbiodatacr website" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/ManuelSpinola/rbiodatacr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ManuelSpinola/rbiodatacr/actions/workflows/R-CMD-check.yaml)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html)
<!-- badges: end -->

`rbiodatacr` is an R client for querying
[BIODATACR](https://biodiversidad.go.cr), the national biodiversity
information platform of Costa Rica managed by the Technical Office of
CONAGEBIO (Comision Nacional para la Gestion de la Biodiversidad, Costa
Rica).

## Installation

``` r
remotes::install_github("ManuelSpinola/rbiodatacr")
```

## Main functions

| Function                   | Description                                   |
|----------------------------|-----------------------------------------------|
| `bdcr_count()`             | Count available records for a taxon           |
| `bdcr_count_batch()`       | Count records for multiple taxa               |
| `bdcr_occurrences()`       | Download occurrence records for a taxon       |
| `bdcr_occurrences_batch()` | Download occurrence records for multiple taxa |
| `bdcr_species_search()`    | Search taxonomic information in the BIE index |
| `bdcr_quality_check()`     | Evaluate record quality and assign flags      |

## Basic usage

``` r
library(rbiodatacr)

# Check data availability
bdcr_count("Panthera onca")
```

``` r
# Download occurrence records
df <- bdcr_occurrences("Panthera onca", rows = 50)
dplyr::glimpse(df)
```

``` r
# Query for multiple species
species <- c("Tapirus bairdii", "Panthera onca", "Ara ambiguus")
counts  <- bdcr_count_batch(species)
counts
```

``` r
# Quality control
df_qc <- bdcr_quality_check(df)
dplyr::count(df_qc, quality_flag, sort = TRUE)
```

## Complete workflow

``` r
library(rbiodatacr)
library(dplyr)

# 1. Explore data availability
species <- c("Tapirus bairdii", "Panthera onca",
             "Ara ambiguus",    "Bradypus variegatus")

counts <- bdcr_count_batch(species)

# 2. Download species with sufficient data
with_data <- filter(counts, n_records >= 10)

occ_list <- bdcr_occurrences_batch(
  taxa = with_data$taxon,
  rows = 200
)

# 3. Quality control and consolidate
df_final <- purrr::map(occ_list, bdcr_quality_check) |>
  bind_rows(.id = "taxon") |>
  filter(quality_flag == "ok",
         !is.na(decimalLatitude),
         !is.na(decimalLongitude))

# 4. Summary
df_final |>
  count(taxon, sort = TRUE) |>
  rename(clean_records = n)
```

## About BIODATACR

BIODATACR is built on the infrastructure of the [Atlas of Living
Australia (ALA)](https://www.ala.org.au/).

## License

MIT © Manuel Spinola

## Logo

Tapir silhouette by [Gabriela
Palomo-Munoz](https://www.phylopic.org/images/eade2272-a39d-4554-810e-8be371334192/tapirus-bairdii)
via [PhyloPic](https://www.phylopic.org/), licensed under [CC BY
3.0](https://creativecommons.org/licenses/by/3.0/).

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  message  = FALSE,
  warning  = FALSE,
  eval     = FALSE
)

## ----setup, eval=TRUE---------------------------------------------------------
library(rbiodatacr)
library(dplyr)
library(sf)
library(ggplot2)

## ----species-search-----------------------------------------------------------
# bdcr_species_search("Panthera onca")

## ----count-single-------------------------------------------------------------
# bdcr_count("Panthera onca")

## ----count-batch--------------------------------------------------------------
# species <- c(
#   "Tapirus bairdii",
#   "Panthera onca",
#   "Ara ambiguus",
#   "Bradypus variegatus"
# )
# 
# conteos <- bdcr_count_batch(species)
# conteos

## ----occurrences-single-------------------------------------------------------
# df_jaguar <- bdcr_occurrences("Panthera onca", rows = 100)
# glimpse(df_jaguar)

## ----occurrences-batch--------------------------------------------------------
# spp_with_data <- filter(conteos, n_records >= 10)
# 
# lista_occ <- bdcr_occurrences_batch(
#   taxa = spp_with_data$taxon,
#   rows = 100
# )
# 
# # Number of records per species
# purrr::map_int(lista_occ, nrow)

## ----quality-check------------------------------------------------------------
# df_qc <- bdcr_quality_check(df_jaguar)
# 
# count(df_qc, quality_flag, sort = TRUE)

## ----quality-filter-----------------------------------------------------------
# df_clean <- filter(df_qc, quality_flag == "ok",
#                          !is.na(decimalLatitude),
#                          !is.na(decimalLongitude))
# nrow(df_clean)

## ----map, fig.width = 7, fig.height = 6---------------------------------------
# # Convert to sf
# df_sf <- st_as_sf(
#   df_clean,
#   coords = c("decimalLongitude", "decimalLatitude"),
#   crs    = 4326
# )
# 
# # Load Costa Rica national boundary included in rbiodatacr
# # Source: GADM (gadm.org), level 0 = country boundary
# data(cr_outline)
# 
# # Map
# ggplot() +
#   geom_sf(data = cr_outline, fill = "gray95", color = "gray50") +
#   geom_sf(data = df_sf, color = "#E63946", size = 2, alpha = 0.7) +
#   labs(
#     title    = "Panthera onca — BIODATACR occurrence records",
#     subtitle = paste0(nrow(df_sf), " clean records"),
#     caption  = "Source: BIODATACR (biodiversidad.go.cr)",
#     x = "Longitude",
#     y = "Latitude"
#   ) +
#   theme_minimal()

## ----workflow-----------------------------------------------------------------
# # 1. Check availability
# species <- c("Tapirus bairdii", "Panthera onca",
#              "Ara ambiguus",    "Bradypus variegatus")
# 
# conteos <- bdcr_count_batch(species)
# 
# # 2. Download species with enough data
# con_datos <- filter(conteos, n_records >= 10)
# 
# lista_occ <- bdcr_occurrences_batch(
#   taxa = con_datos$taxon,
#   rows = 200
# )
# 
# # 3. Quality control
# lista_limpia <- purrr::map(lista_occ, bdcr_quality_check)
# 
# # 4. Consolidate and filter
# df_final <- bind_rows(lista_limpia, .id = "taxon") |>
#   filter(quality_flag == "ok",
#          !is.na(decimalLatitude),
#          !is.na(decimalLongitude))
# 
# # 5. Summary
# df_final |>
#   count(taxon, sort = TRUE) |>
#   rename(clean_records = n)

#' Download occurrence records from BIODATACR
#'
#' @param taxon  Character. Scientific name of the taxon.
#' @param rows   Integer. Maximum number of records to download. Default 100.
#' @param start  Integer. Starting record for pagination. Default 0.
#'
#' @return A `tibble` with columns: `scientificName`, `vernacularName`,
#'   `decimalLatitude`, `decimalLongitude`, `year`, `month`,
#'   `basisOfRecord`, `dataResourceName`, `country`, `family`,
#'   `species`, `collector`, `license`, `geospatialKosher`,
#'   `taxonomicKosher`.
#' @export
#' @examples
#' \dontrun{
#' bdcr_occurrences("Panthera onca", rows = 50)
#' }
bdcr_occurrences <- function(taxon, rows = 100, start = 0) {
  bdcr_check_string(taxon, "taxon")
  bdcr_check_count(rows, "rows")

  url  <- bdcr_url("occurrences/search")
  resp <- bdcr_GET(url, query = list(q = taxon, pageSize = rows, start = start))

  occs <- resp[["occurrences"]]

  if (is.null(occs) || nrow(occs) == 0) {
    cli::cli_inform("No records found for {.val {taxon}}.")
    return(dplyr::tibble())
  }

  occs |>
    dplyr::as_tibble() |>
    dplyr::select(
      scientificName, vernacularName,
      decimalLatitude, decimalLongitude,
      year, month, basisOfRecord, dataResourceName,
      country, family, species, collector, license,
      geospatialKosher, taxonomicKosher
    ) |>
    dplyr::mutate(
      geospatialKosher = geospatialKosher == "true",
      taxonomicKosher  = taxonomicKosher  == "true"
    )
}

#' Download occurrence records for multiple taxa
#'
#' @param taxa  Character vector. Scientific names.
#' @param rows  Integer. Records per taxon. Default 100.
#' @param wait  Numeric. Seconds to pause between requests. Default 1.
#'
#' @return Named list of tibbles, one per taxon.
#' @export
#' @examples
#' \dontrun{
#' spp <- c("Tapirus bairdii", "Panthera onca")
#' bdcr_occurrences_batch(spp, rows = 50)
#' }
bdcr_occurrences_batch <- function(taxa, rows = 100, wait = 1) {
  if (!is.character(taxa) || length(taxa) == 0)
    cli::cli_abort("{.arg taxa} must be a non-empty character vector.")
  bdcr_check_count(rows, "rows")

  resultados <- vector("list", length(taxa))
  names(resultados) <- taxa

  for (i in seq_along(taxa)) {
    cli::cli_progress_step("Downloading {taxa[[i]]} ({i}/{length(taxa)})")
    resultados[[i]] <- bdcr_occurrences(taxa[[i]], rows = rows)
    if (i < length(taxa)) bdcr_polite_wait(wait)
  }

  resultados
}

#' Count occurrence records for a taxon
#'
#' @param taxon Character. Scientific name (e.g. `"Panthera onca"`).
#' @return Integer with the total number of available records.
#' @export
#' @examples
#' \dontrun{
#' bdcr_count("Panthera onca")
#' }
bdcr_count <- function(taxon) {
  bdcr_check_string(taxon, "taxon")
  url  <- bdcr_url("occurrences/search")
  resp <- bdcr_GET(url, query = list(q = taxon, pageSize = 0))
  resp[["totalRecords"]]
}

#' Search for taxonomic information of a species in BIODATACR
#'
#' Queries the BIE (Biodiversity Information Explorer) index of BIODATACR
#' to retrieve taxonomic information for a species.
#'
#' @param name  Character. Scientific name (may be a synonym or partial name).
#' @param rows  Integer. Maximum number of results. Default 10.
#'
#' @return A `tibble` with columns: `name`, `guid`, `commonName`,
#'   `scientificName`, `rank`, `taxonomicStatus`, `nameComplete`.
#' @export
#' @examples
#' \dontrun{
#' bdcr_species_search("Panthera onca")
#' }
bdcr_species_search <- function(name, rows = 10) {
  bdcr_check_string(name, "name")
  bdcr_check_count(rows, "rows")

  url  <- paste0(BDCR_BIE_URL, "/search")
  resp <- bdcr_GET(url, query = list(q = name, pageSize = rows))

  resultados <- resp[["searchResults"]][["results"]]

  if (is.null(resultados) || nrow(resultados) == 0) {
    cli::cli_inform("No results found for {.val {name}}.")
    return(dplyr::tibble())
  }

  resultados |>
    dplyr::as_tibble() |>
    dplyr::select(
      name, guid, commonName, scientificName,
      rank, taxonomicStatus, nameComplete
    )
}

# R/utils.R
# Internal infrastructure functions for rbiodatacr
# Not exported — internal use only

# Suppress R CMD check notes for dplyr column names
utils::globalVariables(c(
  "scientificName", "vernacularName",
  "decimalLatitude", "decimalLongitude",
  "year", "month", "basisOfRecord", "dataResourceName",
  "country", "family", "species", "collector", "license",
  "geospatialKosher", "taxonomicKosher",
  "guid", "commonName", "taxonomicStatus", "nameComplete",
  "quality_flag", "n_records"
))

# -- Constants -----------------------------------------------------------------


BDCR_BASE_URL <- "http://datos.biodiversidad.go.cr/biocache-service"
BDCR_BIE_URL <- "http://datos.biodiversidad.go.cr/bie-index"

# -- URL builder ---------------------------------------------------------------

#' Build full URL for a BIODATACR biocache endpoint
#'
#' @param endpoint Character. Relative path, e.g. `"occurrences/search"`.
#' @return Character with the full URL.
#' @noRd
bdcr_url <- function(endpoint) {
  paste0(BDCR_BASE_URL, "/", endpoint)
}

# -- HTTP client ---------------------------------------------------------------

#' Perform a GET request to the BIODATACR API
#'
#' Wraps `httr::GET()` with network error handling, HTTP status checking,
#' and automatic JSON parsing.
#'
#' @param url     Character. Full endpoint URL.
#' @param query   Named list of query parameters. Default `list()`.
#' @param timeout Numeric. Seconds before aborting the connection. Default `60`.
#'
#' @return Parsed list from the JSON response.
#' @noRd
bdcr_GET <- function(url, query = list(), timeout = 60) {

  # Request — catch network errors before they propagate
  resp <- tryCatch(
    httr::GET(
      url,
      query = query,
      httr::timeout(timeout),
      httr::user_agent(
        "rbiodatacr/0.1.0 (https://github.com/ManuelSpinola/rbiodatacr)"
      )
    ),
    error = function(e) {
      cli::cli_abort(
        c("Could not connect to BIODATACR.",
          "i" = "Please check your internet connection.",
          "x" = conditionMessage(e)),
        call = NULL
      )
    }
  )

  # Check HTTP status code
  status <- httr::status_code(resp)
  if (status != 200L) {
    cli::cli_abort(
      c("BIODATACR server returned HTTP {status}.",
        "i" = "URL: {url}"),
      call = NULL
    )
  }

  # Check content-type
  ct <- httr::headers(resp)[["content-type"]]
  if (!grepl("json", ct, ignore.case = TRUE)) {
    cli::cli_abort(
      c("Response is not JSON (content-type: {ct}).",
        "i" = "URL: {url}"),
      call = NULL
    )
  }

  # Parse JSON — simplifyVector = TRUE returns data.frames for tabular fields
  raw_text <- httr::content(resp, as = "text", encoding = "UTF-8")
  parsed   <- tryCatch(
    jsonlite::fromJSON(raw_text, simplifyVector = TRUE),
    error = function(e) {
      cli::cli_abort(
        c("Could not parse BIODATACR JSON response.",
          "x" = conditionMessage(e)),
        call = NULL
      )
    }
  )

  parsed
}

# -- Polite pause --------------------------------------------------------------

#' Pause between API requests
#'
#' Adds a pause of at least `seconds` seconds to avoid overloading the server.
#'
#' @param seconds Numeric. Seconds to wait. Default `1`.
#' @noRd
bdcr_polite_wait <- function(seconds = 1) {
  Sys.sleep(seconds)
}

# -- Argument validation -------------------------------------------------------

#' Check that an argument is a non-empty character string
#'
#' @param x   Object to check.
#' @param arg Argument name (used in error message).
#' @noRd
bdcr_check_string <- function(x, arg = deparse(substitute(x))) {
  if (!is.character(x) || length(x) != 1L || nchar(trimws(x)) == 0L) {
    cli::cli_abort(
      "{.arg {arg}} must be a non-empty character string of length 1.",
      call = NULL
    )
  }
  invisible(x)
}

#' Check that an argument is a positive integer
#'
#' @param x   Object to check.
#' @param arg Argument name (used in error message).
#' @noRd
bdcr_check_count <- function(x, arg = deparse(substitute(x))) {
  if (!is.numeric(x) || length(x) != 1L || x < 1L || x != as.integer(x)) {
    cli::cli_abort(
      "{.arg {arg}} must be a positive integer.",
      call = NULL
    )
  }
  invisible(as.integer(x))
}

test_that("bdcr_url construye URLs correctamente", {
  expect_equal(
    rbiodatacr:::bdcr_url("occurrences/search"),
    "http://datos.biodiversidad.go.cr/biocache-service/occurrences/search"
  )
})

test_that("bdcr_check_string rechaza valores inválidos", {
  expect_error(rbiodatacr:::bdcr_check_string(""))
  expect_error(rbiodatacr:::bdcr_check_string(123))
  expect_error(rbiodatacr:::bdcr_check_string(NA_character_))
  expect_silent(rbiodatacr:::bdcr_check_string("Panthera onca"))
})

test_that("bdcr_check_count rechaza valores inválidos", {
  expect_error(rbiodatacr:::bdcr_check_count(0))
  expect_error(rbiodatacr:::bdcr_check_count(-5))
  expect_error(rbiodatacr:::bdcr_check_count("10"))
  expect_silent(rbiodatacr:::bdcr_check_count(100))
})

#' Costa Rica national boundary
#'
#' An `sf` object with the national boundary of Costa Rica from GADM.
#'
#' @format An `sf` object with 1 row and the following columns:
#' \describe{
#'   \item{GID_0}{Country code (CRI)}
#'   \item{COUNTRY}{Country name}
#'   \item{geometry}{MULTIPOLYGON geometry}
#' }
#' @source \url{https://gadm.org/}
"cr_outline"

#' Count occurrence records for multiple taxa
#'
#' @param taxa Character vector. Scientific names.
#' @param wait Numeric. Seconds to pause between requests. Default 1.
#'
#' @return A `tibble` with columns `taxon` and `n_records`.
#' @export
#' @examples
#' \dontrun{
#' spp <- c("Tapirus bairdii", "Panthera onca")
#' bdcr_count_batch(spp)
#' }
bdcr_count_batch <- function(taxa, wait = 1) {
  if (!is.character(taxa) || length(taxa) == 0)
    cli::cli_abort("{.arg taxa} must be a non-empty character vector.")

  purrr::imap_dfr(taxa, function(sp, i) {
    cli::cli_progress_step("Counting {sp} ({i}/{length(taxa)})")
    n <- tryCatch(bdcr_count(sp), error = function(e) NA_integer_)
    if (i < length(taxa)) bdcr_polite_wait(wait)
    dplyr::tibble(taxon = sp, n_records = n)
  })
}

#' Quality check for occurrence records
#'
#' Adds a `quality_flag` column to an occurrence tibble. Requires that
#' `geospatialKosher` and `taxonomicKosher` are logical — a condition
#' guaranteed by `bdcr_occurrences()`.
#'
#' @param df       A `tibble` of occurrence records (output of `bdcr_occurrences()`).
#' @param min_year Integer. Minimum acceptable year. Default 1950.
#'
#' @return The same `tibble` with an additional `quality_flag` column.
#'   Possible values:
#'   \describe{
#'     \item{`"ok"`}{No issues detected.}
#'     \item{`"no_coords"`}{Missing coordinates.}
#'     \item{`"geospatial_issue"`}{`geospatialKosher == FALSE`.}
#'     \item{`"taxonomic_issue"`}{`taxonomicKosher == FALSE`.}
#'     \item{`"old_record"`}{Year before `min_year`.}
#'   }
#' @export
#' @examples
#' \dontrun{
#' df <- bdcr_occurrences("Panthera onca", rows = 50)
#' bdcr_quality_check(df)
#' }
bdcr_quality_check <- function(df, min_year = 1950) {
  if (!inherits(df, "data.frame") || nrow(df) == 0)
    cli::cli_abort("{.arg df} must be a non-empty data.frame.")

  df |>
    dplyr::mutate(
      quality_flag = dplyr::case_when(
        is.na(decimalLatitude) | is.na(decimalLongitude) ~ "no_coords",
        geospatialKosher == FALSE                         ~ "geospatial_issue",
        taxonomicKosher  == FALSE                         ~ "taxonomic_issue",
        !is.na(year) & year < min_year                   ~ "old_record",
        TRUE                                              ~ "ok"
      )
    )
}

test_that("bdcr_quality_check etiqueta coordenadas ausentes", {
  df <- dplyr::tibble(
    decimalLatitude  = NA_real_,
    decimalLongitude = NA_real_,
    geospatialKosher = TRUE,
    taxonomicKosher  = TRUE,
    year             = 2020
  )
  expect_equal(bdcr_quality_check(df)$quality_flag, "no_coords")
})

test_that("bdcr_quality_check marca registros limpios como ok", {
  df <- dplyr::tibble(
    decimalLatitude  = 9.5,
    decimalLongitude = -84.0,
    geospatialKosher = TRUE,
    taxonomicKosher  = TRUE,
    year             = 2020
  )
  expect_equal(bdcr_quality_check(df)$quality_flag, "ok")
})

---
title: "Introduction to rbiodatacr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to rbiodatacr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  message  = FALSE,
  warning  = FALSE,
  eval     = FALSE
)
```

## Overview

`rbiodatacr` is an R client for querying
[BIODATACR](https://biodiversidad.go.cr), the national biodiversity
information platform of Costa Rica managed by the Technical Office of
CONAGEBIO. The platform is built on the
[Atlas of Living Australia (ALA)](https://www.ala.org.au/) API
infrastructure.

```{r setup, eval=TRUE}
library(rbiodatacr)
library(dplyr)
library(sf)
library(ggplot2)
```

---

## 1. Taxonomic search

Before downloading occurrence records, use `bdcr_species_search()` to
verify that the species name is recognized by BIODATACR and to retrieve
its taxonomic identifier (GUID).

```{r species-search}
bdcr_species_search("Panthera onca")
```

The function may return more than one row when both the species and
subspecies are registered. The `guid` column contains the unique
identifier for each taxonomic concept — useful for precise queries.

---

## 2. Counting records

Use `bdcr_count()` to check how many occurrence records are available
before downloading.

```{r count-single}
bdcr_count("Panthera onca")
```

For multiple species at once use `bdcr_count_batch()`, which returns a
tidy tibble with one row per species.

```{r count-batch}
species <- c(
  "Tapirus bairdii",
  "Panthera onca",
  "Ara ambiguus",
  "Bradypus variegatus"
)

conteos <- bdcr_count_batch(species)
conteos
```

---

## 3. Downloading occurrence records

`bdcr_occurrences()` downloads records for a single species and returns
a tibble with 15 fields relevant for biodiversity analysis.

```{r occurrences-single}
df_jaguar <- bdcr_occurrences("Panthera onca", rows = 100)
glimpse(df_jaguar)
```

For multiple species use `bdcr_occurrences_batch()`, which returns a
named list of tibbles — one per species.

```{r occurrences-batch}
spp_with_data <- filter(conteos, n_records >= 10)

lista_occ <- bdcr_occurrences_batch(
  taxa = spp_with_data$taxon,
  rows = 100
)

# Number of records per species
purrr::map_int(lista_occ, nrow)
```

---

## 4. Quality control

`bdcr_quality_check()` adds a `quality_flag` column to the occurrences
tibble. Possible flags are:

| Flag | Condition |
|---|---|
| `"ok"` | No issues detected |
| `"no_coords"` | Missing coordinates |
| `"geospatial_issue"` | `geospatialKosher == FALSE` |
| `"taxonomic_issue"` | `taxonomicKosher == FALSE` |
| `"old_record"` | Year before minimum threshold (default 1950) |

```{r quality-check}
df_qc <- bdcr_quality_check(df_jaguar)

count(df_qc, quality_flag, sort = TRUE)
```

Keep only clean records:

```{r quality-filter}
df_clean <- filter(df_qc, quality_flag == "ok",
                         !is.na(decimalLatitude),
                         !is.na(decimalLongitude))
nrow(df_clean)
```

---

## 5. Mapping occurrence records

Convert the clean tibble to an `sf` object and plot the records over
Costa Rica.

```{r map, fig.width = 7, fig.height = 6}
# Convert to sf
df_sf <- st_as_sf(
  df_clean,
  coords = c("decimalLongitude", "decimalLatitude"),
  crs    = 4326
)

# Load Costa Rica national boundary included in rbiodatacr
# Source: GADM (gadm.org), level 0 = country boundary
data(cr_outline)

# Map
ggplot() +
  geom_sf(data = cr_outline, fill = "gray95", color = "gray50") +
  geom_sf(data = df_sf, color = "#E63946", size = 2, alpha = 0.7) +
  labs(
    title    = "Panthera onca — BIODATACR occurrence records",
    subtitle = paste0(nrow(df_sf), " clean records"),
    caption  = "Source: BIODATACR (biodiversidad.go.cr)",
    x = "Longitude",
    y = "Latitude"
  ) +
  theme_minimal()
```

---

## 6. Complete workflow

```{r workflow}
# 1. Check availability
species <- c("Tapirus bairdii", "Panthera onca",
             "Ara ambiguus",    "Bradypus variegatus")

conteos <- bdcr_count_batch(species)

# 2. Download species with enough data
con_datos <- filter(conteos, n_records >= 10)

lista_occ <- bdcr_occurrences_batch(
  taxa = con_datos$taxon,
  rows = 200
)

# 3. Quality control
lista_limpia <- purrr::map(lista_occ, bdcr_quality_check)

# 4. Consolidate and filter
df_final <- bind_rows(lista_limpia, .id = "taxon") |>
  filter(quality_flag == "ok",
         !is.na(decimalLatitude),
         !is.na(decimalLongitude))

# 5. Summary
df_final |>
  count(taxon, sort = TRUE) |>
  rename(clean_records = n)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bdcr_count_batch.R
\name{bdcr_count_batch}
\alias{bdcr_count_batch}
\title{Count occurrence records for multiple taxa}
\usage{
bdcr_count_batch(taxa, wait = 1)
}
\arguments{
\item{taxa}{Character vector. Scientific names.}

\item{wait}{Numeric. Seconds to pause between requests. Default 1.}
}
\value{
A `tibble` with columns `taxon` and `n_records`.
}
\description{
Count occurrence records for multiple taxa
}
\examples{
\dontrun{
spp <- c("Tapirus bairdii", "Panthera onca")
bdcr_count_batch(spp)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bdcr_occurrences.R
\name{bdcr_occurrences}
\alias{bdcr_occurrences}
\title{Download occurrence records from BIODATACR}
\usage{
bdcr_occurrences(taxon, rows = 100, start = 0)
}
\arguments{
\item{taxon}{Character. Scientific name of the taxon.}

\item{rows}{Integer. Maximum number of records to download. Default 100.}

\item{start}{Integer. Starting record for pagination. Default 0.}
}
\value{
A `tibble` with columns: `scientificName`, `vernacularName`,
  `decimalLatitude`, `decimalLongitude`, `year`, `month`,
  `basisOfRecord`, `dataResourceName`, `country`, `family`,
  `species`, `collector`, `license`, `geospatialKosher`,
  `taxonomicKosher`.
}
\description{
Download occurrence records from BIODATACR
}
\examples{
\dontrun{
bdcr_occurrences("Panthera onca", rows = 50)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bdcr_occurrences_batch.R
\name{bdcr_occurrences_batch}
\alias{bdcr_occurrences_batch}
\title{Download occurrence records for multiple taxa}
\usage{
bdcr_occurrences_batch(taxa, rows = 100, wait = 1)
}
\arguments{
\item{taxa}{Character vector. Scientific names.}

\item{rows}{Integer. Records per taxon. Default 100.}

\item{wait}{Numeric. Seconds to pause between requests. Default 1.}
}
\value{
Named list of tibbles, one per taxon.
}
\description{
Download occurrence records for multiple taxa
}
\examples{
\dontrun{
spp <- c("Tapirus bairdii", "Panthera onca")
bdcr_occurrences_batch(spp, rows = 50)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bdcr_count.R
\name{bdcr_count}
\alias{bdcr_count}
\title{Count occurrence records for a taxon}
\usage{
bdcr_count(taxon)
}
\arguments{
\item{taxon}{Character. Scientific name (e.g. `"Panthera onca"`).}
}
\value{
Integer with the total number of available records.
}
\description{
Count occurrence records for a taxon
}
\examples{
\dontrun{
bdcr_count("Panthera onca")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bdcr_quality_check.R
\name{bdcr_quality_check}
\alias{bdcr_quality_check}
\title{Quality check for occurrence records}
\usage{
bdcr_quality_check(df, min_year = 1950)
}
\arguments{
\item{df}{A `tibble` of occurrence records (output of `bdcr_occurrences()`).}

\item{min_year}{Integer. Minimum acceptable year. Default 1950.}
}
\value{
The same `tibble` with an additional `quality_flag` column.
  Possible values:
  \describe{
    \item{`"ok"`}{No issues detected.}
    \item{`"no_coords"`}{Missing coordinates.}
    \item{`"geospatial_issue"`}{`geospatialKosher == FALSE`.}
    \item{`"taxonomic_issue"`}{`taxonomicKosher == FALSE`.}
    \item{`"old_record"`}{Year before `min_year`.}
  }
}
\description{
Adds a `quality_flag` column to an occurrence tibble. Requires that
`geospatialKosher` and `taxonomicKosher` are logical — a condition
guaranteed by `bdcr_occurrences()`.
}
\examples{
\dontrun{
df <- bdcr_occurrences("Panthera onca", rows = 50)
bdcr_quality_check(df)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bdcr_species_search.R
\name{bdcr_species_search}
\alias{bdcr_species_search}
\title{Search for taxonomic information of a species in BIODATACR}
\usage{
bdcr_species_search(name, rows = 10)
}
\arguments{
\item{name}{Character. Scientific name (may be a synonym or partial name).}

\item{rows}{Integer. Maximum number of results. Default 10.}
}
\value{
A `tibble` with columns: `name`, `guid`, `commonName`,
  `scientificName`, `rank`, `taxonomicStatus`, `nameComplete`.
}
\description{
Queries the BIE (Biodiversity Information Explorer) index of BIODATACR
to retrieve taxonomic information for a species.
}
\examples{
\dontrun{
bdcr_species_search("Panthera onca")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cr_outline}
\alias{cr_outline}
\title{Costa Rica national boundary}
\format{
An `sf` object with 1 row and the following columns:
\describe{
  \item{GID_0}{Country code (CRI)}
  \item{COUNTRY}{Country name}
  \item{geometry}{MULTIPOLYGON geometry}
}
}
\source{
\url{https://gadm.org/}
}
\usage{
cr_outline
}
\description{
An `sf` object with the national boundary of Costa Rica from GADM.
}
\keyword{datasets}
